@@ -3,7 +3,7 @@ from pyx import *
 
 class daystyle(graph.style._style):
 
-    def columnnames(self, privatedata, sharedata, graph, columnnames):
+    def columnnames(self, privatedata, sharedata, graph, columnnames, dataaxisnames):
         # register the new column names
         usecolumnnames = ["day", "month", "weekday", "note"]
         for columnname in usecolumnnames:
@@ -11,7 +11,7 @@ class daystyle(graph.style._style):
                 raise ValueError("column '%s' missing" % columnname)
         return usecolumnnames
 
-    def adjustaxis(self, privatedata, sharedata, graph, columnname, data):
+    def adjustaxis(self, privatedata, sharedata, graph, plotitem, columnname, data):
         # adjust axes ranges
         if columnname == "month":
             graph.axes["x"].adjustaxis([(x, 0) for x in data])
@@ -36,23 +36,24 @@ class daystyle(graph.style._style):
             graph.text_pt(x1_pt+3, y1_pt+3, point["note"], [text.size.tiny])
 
 # create calendar data
-year = 2006
+year = 2012
 notes = {1: {17: r"\PyX{} 0.2 (2003)", 20: r"\PyX{} 0.5 (2004)", 22: r"\PyX{} 0.5.1 (2004)"},
          3: {30: r"\PyX{} 0.6 (2004)", 31: r"\PyX{} 0.3 ('03), \PyX{} 0.6.1 ('04)"},
          4: {4: r"\PyX{} 0.3.1 (2003)", 7: r"\PyX{} 0.6.2 (2004)", 27: r"\PyX{} 0.6.3 (2004)"},
-         5: {24: r"\PyX{} 0.9 (2006)"},
+         5: {5: r"\PyX{} 0.11 (2011)", 20: r"\PyX{} 0.11.1 (2011)", 24: r"\PyX{} 0.9 (2006)"},
          7: {13: r"\PyX{} 0.8 (2005)"},
          8: {13: r"\PyX{} 0.8.1 (2005)", 22: r"\PyX{} 0.4 (2003)"},
          9: {17: r"\PyX{} 0.4.1 (2003)"},
-         10: {7: r"\PyX{} 0.1 (2002)", 21: r"\PyX{} 0.7 (2004)"},
-         12: {15: r"\PyX{} 0.7.1 (2004)"}}
-d = graph.data.list([(day,
-                      calendar.month_name[month],
-                      calendar.day_abbr[calendar.weekday(year, month, day)],
-                      notes.get(month, {}).get(day))
-                     for month in range(1, 13)
-                     for day in range(1, calendar.monthrange(year, month)[1]+1)],
-                    day=1, month=2, weekday=3, note=4)
+         10: {3: r"\PyX{} 0.10 (2007)", 7: r"\PyX{} 0.1 (2002)", 12: r"\PyX{} 0.12 (2012)",
+              21: r"\PyX{} 0.7 (2004)", 26: r"\PyX{} 0.12.1 (2012)"},
+         12: {15: r"\PyX{} 0.7.1 (2004)", 20: r"\PyX{} 0.13 (2013)"}}
+d = graph.data.points([(day,
+                        calendar.month_name[month],
+                        calendar.day_abbr[calendar.weekday(year, month, day)],
+                        notes.get(month, {}).get(day))
+                       for month in range(1, 13)
+                       for day in range(1, calendar.monthrange(year, month)[1]+1)],
+                      day=1, month=2, weekday=3, note=4)
 
 # create the calendar
 g = graph.graphxy(width=40, x2=graph.axis.bar(dist=0, linkpainter=None),
