@@ -25,22 +25,24 @@
 A canvas holds a collection of all elements and corresponding attributes to be
 displayed. """
 
-import cStringIO, os, sys, string, tempfile, warnings
-import attr, canvasitem, deco, deformer, document, font, pycompat, style, trafo
-import bbox as bboxmodule
+import io, logging, os, sys, string, tempfile
+from . import attr, baseclasses, config, document, style, trafo
+from . import bbox as bboxmodule
+
+logger = logging.getLogger("pyx")
 
 def _wrappedindocument(method):
     def wrappedindocument(self, file=None, **kwargs):
         page_kwargs = {}
         write_kwargs = {}
-        for name, value in kwargs.items():
+        for name, value in list(kwargs.items()):
             if name.startswith("page_"):
                 page_kwargs[name[5:]] = value
             elif name.startswith("write_"):
                 write_kwargs[name[6:]] = value
             else:
-                warnings.warn("Keyword argument %s of %s method should be prefixed with 'page_'" %
-                                (name, method.__name__), DeprecationWarning)
+                logger.warning("implicit page keyword argument passing is deprecated; keyword argument '%s' of %s method should be changed to 'page_%s'" %
+                               (name, method.__name__, name))
                 page_kwargs[name] = value
         d = document.document([document.page(self, **page_kwargs)])
         self.__name__ = method.__name__
@@ -52,7 +54,7 @@ def _wrappedindocument(method):
 # clipping class
 #
 
-class clip(canvasitem.canvasitem):
+class clip(attr.attr):
 
     """class for use in canvas constructor which clips to a path"""
 
@@ -60,20 +62,12 @@ class clip(canvasitem.canvasitem):
         """construct a clip instance for a given path"""
         self.path = path
 
-    def bbox(self):
-        # as a canvasitem a clipping path has NO influence on the bbox...
-        return bboxmodule.empty()
-
-    def clipbbox(self):
-        # ... but for clipping, we nevertheless need the bbox
-        return self.path.bbox()
-
-    def processPS(self, file, writer, context, registry, bbox):
+    def processPS(self, file, writer, context, registry):
         file.write("newpath\n")
         self.path.outputPS(file, writer)
         file.write("clip\n")
 
-    def processPDF(self, file, writer, context, registry, bbox):
+    def processPDF(self, file, writer, context, registry):
         self.path.outputPDF(file, writer)
         file.write("W n\n")
 
@@ -82,7 +76,7 @@ class clip(canvasitem.canvasitem):
 # general canvas class
 #
 
-class canvas(canvasitem.canvasitem):
+class canvas(baseclasses.canvasitem):
 
     """a canvas holds a collection of canvasitems"""
 
@@ -105,8 +99,8 @@ class canvas(canvasitem.canvasitem):
         """
 
         self.items = []
-        self.trafo = trafo.trafo()
-        self.clipbbox = None
+        self.trafo = trafo.identity
+        self.clip = None
         self.layers = {}
         if attrs is None:
             attrs = []
@@ -114,24 +108,26 @@ class canvas(canvasitem.canvasitem):
             self.texrunner = texrunner
         else:
             # prevent cyclic imports
-            import text
+            from . import text
             self.texrunner = text.defaulttexrunner
 
-        attr.checkattrs(attrs, [trafo.trafo_pt, clip, style.strokestyle, style.fillstyle])
-        # We have to reverse the trafos such that the PostScript concat operators
-        # are in the right order. Correspondingly, we below multiply the current self.trafo
-        # from the right.
-        # Note that while for the stroke and fill styles the order doesn't matter at all,
-        # this is not true for the clip operation.
-        for aattr in attrs[::-1]:
+        attr.checkattrs(attrs, [trafo.trafo_pt, clip, style.style])
+        attrs = attr.mergeattrs(attrs)
+        self.modifies_state = bool(attrs)
+
+        self.styles = attr.getattrs(attrs, [style.style])
+
+        # trafos (and one possible clip operation) are applied from left to
+        # right in the attrs list -> reverse for calculating total trafo
+        for aattr in reversed(attr.getattrs(attrs, [trafo.trafo_pt, clip])):
             if isinstance(aattr, trafo.trafo_pt):
                 self.trafo = self.trafo * aattr
-            elif isinstance(aattr, clip):
-                if self.clipbbox is None:
-                    self.clipbbox = aattr.clipbbox().transformed(self.trafo)
-                else:
-                    self.clippbox *= aattr.clipbbox().transformed(self.trafo)
-            self.items.append(aattr)
+                if self.clip is not None:
+                    self.clip = clip(self.clip.path.transformed(aattr))
+            else:
+                if self.clip is not None:
+                    raise ValueError("single clipping allowed only")
+                self.clip = aattr
 
     def __len__(self):
         return len(self.items)
@@ -143,7 +139,7 @@ class canvas(canvasitem.canvasitem):
         """
         Automatically represent as PNG graphic when evaluated in IPython notebook.
         """
-        return self.pipeGS(device="png16m", seekable=True).getvalue()
+        return self.pipeGS(device="png16m").getvalue()
 
     def bbox(self):
         """returns bounding box of canvas
@@ -159,54 +155,71 @@ class canvas(canvasitem.canvasitem):
         # intersect with clipping bounding box (which has already been
         # transformed in canvas.__init__())
         obbox.transform(self.trafo)
-        if self.clipbbox is not None:
-            obbox *= self.clipbbox
+        if self.clip is not None:
+            obbox *= self.clip.path.bbox()
         return obbox
 
     def processPS(self, file, writer, context, registry, bbox):
         context = context()
         if self.items:
-            file.write("gsave\n")
+            if self.modifies_state:
+                file.write("gsave\n")
+                for attr in self.styles:
+                    attr.processPS(file, writer, context, registry)
+                if self.trafo is not trafo.identity:
+                    self.trafo.processPS(file, writer, context, registry)
+                if self.clip is not None:
+                    self.clip.processPS(file, writer, context, registry)
             nbbox = bboxmodule.empty()
             for item in self.items:
                 item.processPS(file, writer, context, registry, nbbox)
             # update bounding bbox
             nbbox.transform(self.trafo)
-            if self.clipbbox is not None:
-                nbbox *= self.clipbbox
+            if self.clip is not None:
+                nbbox *= self.clip.path.bbox()
             bbox += nbbox
-            file.write("grestore\n")
+            if self.modifies_state:
+                file.write("grestore\n")
 
     def processPDF(self, file, writer, context, registry, bbox):
         context = context()
+        textregion = False
         context.trafo = context.trafo * self.trafo
         if self.items:
-            file.write("q\n") # gsave
+            if self.modifies_state:
+                file.write("q\n") # gsave
+                for attr in self.styles:
+                    if isinstance(attr, style.fillstyle):
+                        context.fillstyles.append(attr)
+                    attr.processPDF(file, writer, context, registry)
+                if self.trafo is not trafo.identity:
+                    self.trafo.processPDF(file, writer, context, registry)
+                if self.clip is not None:
+                    self.clip.processPDF(file, writer, context, registry)
             nbbox = bboxmodule.empty()
             for item in self.items:
-                if isinstance(item, style.fillstyle):
-                    context.fillstyles.append(item)
                 if not writer.text_as_path:
-                    if isinstance(item, font.text_pt):
-                        if not context.textregion:
+                    if item.requiretextregion():
+                        if not textregion:
                             file.write("BT\n")
-                            context.textregion = 1
+                            textregion = True
                     else:
-                        if context.textregion:
+                        if textregion:
                             file.write("ET\n")
-                            context.textregion = 0
+                            textregion = False
                             context.selectedfont = None
                 item.processPDF(file, writer, context, registry, nbbox)
-            if context.textregion:
+            if textregion:
                 file.write("ET\n")
-                context.textregion = 0
+                textregion = False
                 context.selectedfont = None
             # update bounding bbox
             nbbox.transform(self.trafo)
-            if self.clipbbox is not None:
-                nbbox *= self.clipbbox
+            if self.clip is not None:
+                nbbox *= self.clip.path.bbox()
             bbox += nbbox
-            file.write("Q\n") # grestore
+            if self.modifies_state:
+                file.write("Q\n") # grestore
 
     def layer(self, name, above=None, below=None):
         """create or get a layer with name
@@ -219,44 +232,59 @@ class canvas(canvasitem.canvasitem):
         by means of the parameters above or below.
 
         """
+        if above is not None and below is not None:
+            raise ValueError("above and below cannot be specified at the same time")
         try:
             group, layer = name.split(".", 1)
         except ValueError:
-            if not name in self.layers:
-                self.layers[name] = self.insert(canvas(texrunner=self.texrunner), after=above, before=below)
+            if name in self.layers:
+                if above is not None or below is not None:
+                    # remove for repositioning
+                    self.items.remove(self.layers[name])
+            else:
+                # create new layer
+                self.layers[name] = canvas(texrunner=self.texrunner)
+                if above is None and below is None:
+                    self.items.append(self.layers[name])
+
+            # (re)position layer
+            if above is not None:
+                self.items.insert(self.items.index(self.layers[above])+1, self.layers[name])
+            elif below is not None:
+                self.items.insert(self.items.index(self.layers[below]), self.layers[name])
+
             return self.layers[name]
         else:
             if not group in self.layers:
                 self.layers[group] = self.insert(canvas(texrunner=self.texrunner))
+            if above is not None:
+                abovegroup, above = above.split(".", 1)
+                assert abovegroup == group
+            if below is not None:
+                belowgroup, below = below.split(".", 1)
+                assert belowgroup == group
             return self.layers[group].layer(layer, above=above, below=below)
 
-    def insert(self, item, attrs=None, before=None, after=None):
+    def insert(self, item, attrs=None):
         """insert item in the canvas.
 
         If attrs are passed, a canvas containing the item is inserted applying
-        attrs. If one of before or after is not None, the new item is
+        attrs. If replace is not None, the new item is
         positioned accordingly in the canvas.
 
         returns the item, possibly wrapped in a canvas
 
         """
 
-        if not isinstance(item, canvasitem.canvasitem):
-            raise ValueError("only instances of canvasitem.canvasitem can be inserted into a canvas")
+        if not isinstance(item, baseclasses.canvasitem):
+            raise ValueError("only instances of baseclasses.canvasitem can be inserted into a canvas")
 
         if attrs:
             sc = canvas(attrs)
             sc.insert(item)
             item = sc
 
-        if before is not None:
-            if after:
-                raise ValueError("before and after cannot be specified at the same time")
-            self.items.insert(self.items.index(before), item)
-        elif after is not None:
-            self.items.insert(self.items.index(after)+1, item)
-        else:
-            self.items.append(item)
+        self.items.append(item)
         return item
 
     def draw(self, path, attrs):
@@ -268,16 +296,15 @@ class canvas(canvasitem.canvasitem):
         before drawing the path.
 
         """
-
+        from . import deco
         attrs = attr.mergeattrs(attrs)
-        attr.checkattrs(attrs, [deco.deco, deformer.deformer, style.fillstyle, style.strokestyle, style.fillrule])
+        attr.checkattrs(attrs, [deco.deco, baseclasses.deformer, style.style])
 
-        for adeformer in attr.getattrs(attrs, [deformer.deformer]):
+        for adeformer in attr.getattrs(attrs, [baseclasses.deformer]):
             path = adeformer.deform(path)
 
-        styles = attr.getattrs(attrs, [style.fillstyle, style.strokestyle])
-        fillrule, = attr.getattrs(attrs, [style.fillrule]) or [style.fillrule.nonzero_winding]
-        dp = deco.decoratedpath(path, styles=styles, fillrule=fillrule)
+        styles = attr.getattrs(attrs, [style.style])
+        dp = deco.decoratedpath(path, styles=styles)
 
         # add path decorations and modify path accordingly
         for adeco in attr.getattrs(attrs, [deco.deco]):
@@ -294,7 +321,7 @@ class canvas(canvasitem.canvasitem):
         before drawing the path.
 
         """
-
+        from . import deco
         self.draw(path, [deco.stroked]+list(attrs))
 
     def fill(self, path, attrs=[]):
@@ -306,7 +333,7 @@ class canvas(canvasitem.canvasitem):
         before drawing the path.
 
         """
-
+        from . import deco
         self.draw(path, [deco.filled]+list(attrs))
 
     def settexrunner(self, texrunner):
@@ -339,24 +366,19 @@ class canvas(canvasitem.canvasitem):
     writetofile = _wrappedindocument(document.document.writetofile)
 
 
-    def _gscmd(self, device, filename, resolution=100, gscmd="gs", gsoptions="",
+    def _gscmd(self, device, filename, resolution=100, gs="gs", gsoptions=[],
                textalphabits=4, graphicsalphabits=4, ciecolor=False, **kwargs):
 
-        allowed_chars = string.ascii_letters + string.digits + "_-./"
-        if filename.translate(string.maketrans("", ""), allowed_chars):
-            raise ValueError("for security reasons, only characters, digits and the characters '_-./' are allowed in filenames")
-
-        gscmd += " -dEPSCrop -dNOPAUSE -dQUIET -dBATCH -r%i -sDEVICE=%s -sOutputFile=%s" % (resolution, device, filename)
-        if gsoptions:
-            gscmd += " %s" % gsoptions
+        cmd = [gs, "-dEPSCrop", "-dNOPAUSE", "-dQUIET", "-dBATCH", "-r%d" % resolution, "-sDEVICE=%s" % device, "-sOutputFile=%s" % filename]
         if textalphabits is not None:
-            gscmd += " -dTextAlphaBits=%i" % textalphabits
+            cmd.append("-dTextAlphaBits=%i" % textalphabits)
         if graphicsalphabits is not None:
-            gscmd += " -dGraphicsAlphaBits=%i" % graphicsalphabits
+            cmd.append("-dGraphicsAlphaBits=%i" % graphicsalphabits)
         if ciecolor:
-            gscmd += " -dUseCIEColor"
+            cmd.append("-dUseCIEColor")
+        cmd.extend(gsoptions)
 
-        return gscmd, kwargs
+        return cmd, kwargs
 
     def writeGSfile(self, filename=None, device=None, input="eps", **kwargs):
         """
@@ -386,67 +408,48 @@ class canvas(canvasitem.canvasitem):
             else:
                 raise RuntimeError("could not auto-guess device")
 
-        gscmd, kwargs = self._gscmd(device, filename, **kwargs)
+        cmd, kwargs = self._gscmd(device, filename, **kwargs)
 
         if input == "eps":
-            gscmd += " -"
-            stdin = pycompat.popen(gscmd, "wb")
-            self.writeEPSfile(stdin, **kwargs)
-            stdin.close()
+            cmd.append("-")
+            p = config.Popen(cmd, stdin=config.PIPE)
+            self.writeEPSfile(p.stdin, **kwargs)
+            p.stdin.close()
+            p.wait()
         elif input == "pdf":
             # PDF files need to be accesible by random access and thus we need to create
             # a temporary file
-            fd, fname = tempfile.mkstemp()
-            f = os.fdopen(fd, "wb")
-            gscmd += " %s" % fname
-            self.writePDFfile(f, **kwargs)
-            f.close()
-            os.system(gscmd)
+            with tempfile.NamedTemporaryFile("wb", delete=False) as f:
+                self.writePDFfile(f, **kwargs)
+                fname = f.name
+            cmd.append(fname)
+            config.Popen(cmd).wait()
             os.unlink(fname)
         else:
             raise RuntimeError("input 'eps' or 'pdf' expected")
 
 
-    def pipeGS(self, device, input="eps", seekable=False, **kwargs):
+    def pipeGS(self, device, input="eps", **kwargs):
         """
-        returns a pipe with the Ghostscript output of the EPS or PDF of the canvas
-
-        If seekable is True, a StringIO instance will be returned instead of a
-        pipe to allow random access.
+        returns a BytesIO instance with the Ghostscript output of the EPS or PDF
         """
 
-        gscmd, kwargs = self._gscmd(device, "-", **kwargs)
+        cmd, kwargs = self._gscmd(device, "-", **kwargs)
 
-        if input == "eps":
-            gscmd += " -"
-            # we can safely ignore that the input and output pipes could block each other,
-            # because Ghostscript has to read the full input before writing the output
-            stdin, stdout = pycompat.popen2(gscmd)
-            self.writeEPSfile(stdin, **kwargs)
-            stdin.close()
-        elif input == "pdf":
-            # PDF files need to be accesible by random access and thus we need to create
-            # a temporary file
-            fd, fname = tempfile.mkstemp()
-            f = os.fdopen(fd, "wb")
-            gscmd += " %s" % fname
-            self.writePDFfile(f, **kwargs)
-            f.close()
-            stdout = pycompat.popen(gscmd, "rb")
-            os.unlink(fname)
-        else:
-            raise RuntimeError("input 'eps' or 'pdf' expected")
+        with tempfile.NamedTemporaryFile("wb", delete=False) as f:
+            if input == "eps":
+                self.writeEPSfile(f, **kwargs)
+            elif input == "pdf":
+                self.writePDFfile(f, **kwargs)
+            else:
+                raise RuntimeError("input 'eps' or 'pdf' expected")
+            fname = f.name
 
-        if seekable:
-            # the read method of a pipe object may not return the full content
-            f = cStringIO.StringIO()
-            while True:
-                data = stdout.read()
-                if not data:
-                   break
-                f.write(data)
-            stdout.close()
-            f.seek(0)
-            return f
-        else:
-            return stdout
+        cmd.append(fname)
+        p = config.Popen(cmd, stdout=config.PIPE)
+        data, error = p.communicate()
+        os.unlink(fname)
+
+        if error:
+            raise ValueError("error received while waiting for ghostscript")
+        return io.BytesIO(data)
