@@ -20,14 +20,16 @@
 # along with PyX; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
-import os, string, tempfile, warnings
-import canvasitem, bbox, filelocator, unit, trafo, pswriter
+import logging, os, string, tempfile
+from . import baseclasses, bbox, config, unit, trafo, pswriter
+
+logger = logging.getLogger("pyx")
 
 # PostScript-procedure definitions (cf. 5002.EPSF_Spec_v3.0.pdf)
 # with important correction in EndEPSF:
 #   end operator is missing in the spec!
 
-_BeginEPSF = pswriter.PSdefinition("BeginEPSF", """{
+_BeginEPSF = pswriter.PSdefinition("BeginEPSF", b"""{
   /b4_Inc_state save def
   /dict_count countdictstack def
   /op_count count 1 sub def
@@ -44,7 +46,7 @@ _BeginEPSF = pswriter.PSdefinition("BeginEPSF", """{
   } if
 } bind""")
 
-_EndEPSF = pswriter.PSdefinition("EndEPSF", """{
+_EndEPSF = pswriter.PSdefinition("EndEPSF", b"""{
   end
   count op_count sub {pop} repeat
   countdictstack dict_count sub {end} repeat
@@ -65,7 +67,7 @@ class linefilereader:
     #       in the DSC spec #5001, while '\n\r' *is* a *single* linebreak
     #       according to the EPSF spec #5002
 
-    def __init__(self, filename, typicallinelen=257):
+    def __init__(self, file, typicallinelen=257):
         """Opens the file filename for reading.
 
         typicallinelen defines the default buffer increase
@@ -74,9 +76,8 @@ class linefilereader:
         # note: The maximal line size in an EPS is 255 plus the
         #       linebreak characters. However, we also handle
         #       lines longer than that.
-
-        self.file = open(filename, "rb")
-        self.buffer = ""
+        self.file = file
+        self.buffer = b""
         self.typicallinelen = typicallinelen
 
     def read(self, count=None, EOFmsg="unexpected end of file"):
@@ -113,9 +114,9 @@ class linefilereader:
         an IOError with the EOFmsg message. When EOFmsg is None, an empty
         string is returned when reading beyond the end of the file."""
         EOF = 0
-        while 1:
-            crpos = self.buffer.find("\r")
-            nlpos = self.buffer.find("\n")
+        while True:
+            crpos = self.buffer.find(b"\r")
+            nlpos = self.buffer.find(b"\n")
             if nlpos == -1 and (crpos == -1 or crpos == len(self.buffer) - 1) and not EOF:
                 newbuffer = self.file.read(self.typicallinelen)
                 if not len(newbuffer):
@@ -133,35 +134,31 @@ class linefilereader:
                 self.buffer = self.buffer[eol:]
                 return result
 
-    def close(self):
-        "closes the file"
-        self.file.close()
 
-
-def _readbbox(filename):
+def _readbbox(file):
     """returns bounding box of EPS file filename"""
 
-    file = linefilereader(filename)
+    file = linefilereader(file)
 
     # check the %! header comment
-    if not file.readline().startswith("%!"):
+    if not file.readline().startswith(b"%!"):
         raise IOError("file doesn't start with a '%!' header comment")
 
     bboxatend = 0
     # parse the header (use the first BoundingBox)
-    while 1:
+    while True:
         line = file.readline()
         if not line:
             break
-        if line.startswith("%%BoundingBox:") and not bboxatend:
-            values = line.split(":", 1)[1].split()
+        if line.startswith(b"%%BoundingBox:") and not bboxatend:
+            values = line.split(b":", 1)[1].split()
             if values == ["(atend)"]:
                 bboxatend = 1
             else:
                 if len(values) != 4:
                     raise IOError("invalid number of bounding box values")
-                return bbox.bbox_pt(*map(int, values))
-        elif (line.rstrip() == "%%EndComments" or
+                return bbox.bbox_pt(*list(map(int, values)))
+        elif (line.rstrip() == b"%%EndComments" or
               (len(line) >= 2 and line[0] != "%" and line[1] not in string.whitespace)):
             # implicit end of comments section
             break
@@ -170,17 +167,17 @@ def _readbbox(filename):
 
     # parse the body
     nesting = 0 # allow for nested documents
-    while 1:
+    while True:
         line = file.readline()
-        if line.startswith("%%BeginData:"):
+        if line.startswith(b"%%BeginData:"):
             values = line.split(":", 1)[1].split()
             if len(values) > 3:
                 raise IOError("invalid number of arguments")
             if len(values) == 3:
-                if values[2] == "Lines":
-                    for i in xrange(int(values[0])):
+                if values[2] == b"Lines":
+                    for i in range(int(values[0])):
                         file.readline()
-                elif values[2] != "Bytes":
+                elif values[2] != b"Bytes":
                     raise IOError("invalid bytesorlines-value")
                 else:
                     file.read(int(values[0]))
@@ -190,23 +187,23 @@ def _readbbox(filename):
             # ignore tailing whitespace/newline for binary data
             if (len(values) < 3 or values[2] != "Lines") and not len(line.strip()):
                 line = file.readline()
-            if line.rstrip() != "%%EndData":
+            if line.rstrip() != b"%%EndData":
                 raise IOError("missing EndData")
-        elif line.startswith("%%BeginBinary:"):
+        elif line.startswith(b"%%BeginBinary:"):
             file.read(int(line.split(":", 1)[1]))
             line = file.readline()
             # ignore tailing whitespace/newline
             if not len(line.strip()):
                 line = file.readline()
-            if line.rstrip() != "%%EndBinary":
+            if line.rstrip() != b"%%EndBinary":
                 raise IOError("missing EndBinary")
-        elif line.startswith("%%BeginDocument:"):
+        elif line.startswith(b"%%BeginDocument:"):
             nesting += 1
-        elif line.rstrip() == "%%EndDocument":
+        elif line.rstrip() == b"%%EndDocument":
             if nesting < 1:
                 raise IOError("unmatched EndDocument")
             nesting -= 1
-        elif not nesting and line.rstrip() == "%%Trailer":
+        elif not nesting and line.rstrip() == b"%%Trailer":
             break
 
     usebbox = None
@@ -215,16 +212,16 @@ def _readbbox(filename):
     while line:
         line = file.readline(EOFmsg=None)
         if line.startswith("%%BoundingBox:"):
-            values = line.split(":", 1)[1].split()
+            values = line.split(b":", 1)[1].split()
             if len(values) != 4:
                 raise IOError("invalid number of bounding box values")
-            usebbox = bbox.bbox_pt(*map(int, values))
+            usebbox = bbox.bbox_pt(*list(map(int, values)))
     if not usebbox:
         raise IOError("missing bounding box information in document trailer")
     return usebbox
 
 
-class epsfile(canvasitem.canvasitem):
+class epsfile(baseclasses.canvasitem):
 
     """class for epsfiles"""
 
@@ -248,7 +245,11 @@ class epsfile(canvasitem.canvasitem):
         self.y_pt = unit.topt(y)
         self.filename = filename
         self.kpsearch = kpsearch
-        self.mybbox = bbox or _readbbox(self.filename)
+        if bbox:
+            self.mybbox = bbox
+        else:
+            with self.open() as epsfile:
+                self.mybbox = _readbbox(epsfile)
 
         # determine scaling in x and y direction
         self.scalex = self.scaley = scale
@@ -307,6 +308,12 @@ class epsfile(canvasitem.canvasitem):
         if translatebbox:
             self.trafo = self.trafo * trafo.translate_pt(-self.mybbox.llx_pt, -self.mybbox.lly_pt)
 
+    def open(self):
+        if self.kpsearch:
+            return config.open(self.filename, [config.format.pict])
+        else:
+            return open(self.filename, "rb")
+
     def bbox(self):
         return self.mybbox.transformed(self.trafo)
 
@@ -315,38 +322,30 @@ class epsfile(canvasitem.canvasitem):
         registry.add(_EndEPSF)
         bbox += self.bbox()
 
-        if self.kpsearch:
-            epsfile = filelocator.open(self.filename, [filelocator.format.pict], "rb")
-        else:
-            epsfile = open(self.filename, "rb")
-
         file.write("BeginEPSF\n")
 
         if self.clip:
             llx_pt, lly_pt, urx_pt, ury_pt = self.mybbox.transformed(self.trafo).highrestuple_pt()
             file.write("%g %g %g %g rectclip\n" % (llx_pt, lly_pt, urx_pt-llx_pt, ury_pt-lly_pt))
 
-        self.trafo.processPS(file, writer, context, registry, bbox)
+        self.trafo.processPS(file, writer, context, registry)
 
         file.write("%%%%BeginDocument: %s\n" % self.filename)
-        file.write(epsfile.read())
+
+        with self.open() as epsfile:
+            file.write_bytes(epsfile.read())
+
         file.write("%%EndDocument\n")
         file.write("EndEPSF\n")
-        epsfile.close()
 
     def processPDF(self, file, writer, context, registry, bbox):
-        warnings.warn("EPS file is included as a bitmap created using pipeGS")
+        logger.warning("EPS file is included as a bitmap created using pipeGS")
         from pyx import bitmap, canvas
-        import Image
+        from PIL import Image
         c = canvas.canvas()
         c.insert(self)
-        fd, fname = tempfile.mkstemp()
-        f = os.fdopen(fd, "wb")
-        f.close()
-        c.pipeGS(fname, device="pngalpha", resolution=600)
-        i = Image.open(fname)
+        i = Image.open(c.pipeGS(device="pngalpha", resolution=600))
         i.load()
-        os.unlink(fname)
         b = bitmap.bitmap_pt(self.bbox().llx_pt, self.bbox().lly_pt, i)
         # we slightly shift the bitmap to re-center it, as the bitmap might contain some additional border
         # unfortunately we need to construct another bitmap instance for that ...
