@@ -23,17 +23,13 @@
 
 import math
 from math import cos, sin, tan, acos, pi, radians, degrees
-import trafo, unit
-from normpath import NormpathException, normpath, normsubpath, normline_pt, normcurve_pt
-import bbox as bboxmodule
+from . import trafo, unit
+from .normpath import NormpathException, normpath, normsubpath, normline_pt, normcurve_pt
+from . import bbox as bboxmodule
 
 # set is available as an external interface to the normpath.set method
-from normpath import set
-# normpath's invalid is available as an external interface
-from normpath import invalid
+from .normpath import set
 
-# use new style classes when possible
-__metaclass__ = type
 
 class _marker: pass
 
@@ -1084,20 +1080,17 @@ class path:
             return self._normpath
         if self.pathitems:
             if epsilon is _marker:
-                normpath = self.pathitems[0].createnormpath()
+                np = self.pathitems[0].createnormpath()
             else:
-                normpath = self.pathitems[0].createnormpath(epsilon)
+                np = self.pathitems[0].createnormpath(epsilon)
             context = self.pathitems[0].createcontext()
             for pathitem in self.pathitems[1:]:
-                pathitem.updatenormpath(normpath, context)
+                pathitem.updatenormpath(np, context)
         else:
-            if epsilon is _marker:
-                normpath = normpath([])
-            else:
-                normpath = normpath(epsilon=epsilon)
+            np = normpath()
         if epsilon is _marker:
-            self._normpath = normpath
-        return normpath
+            self._normpath = np
+        return np
 
     def paramtoarclen_pt(self, params):
         """return arc lenght(s) in pts matching the given param(s)"""
