@@ -1,9 +1,8 @@
-#!/usr/bin/env python
-# -*- coding: ISO-8859-1 -*-
+# -*- encoding: utf-8 -*-
 #
 #
-# Copyright (C) 2002-2004 Jrg Lehmann <joergl@users.sourceforge.net>
-# Copyright (C) 2002-2004 Andr Wobst <wobsta@users.sourceforge.net>
+# Copyright (C) 2002-2011 Jörg Lehmann <joergl@users.sourceforge.net>
+# Copyright (C) 2002-2011 André Wobst <wobsta@users.sourceforge.net>
 #
 # This file is part of PyX (http://pyx.sourceforge.net/).
 #
@@ -19,10 +18,10 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with PyX; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
 import math
-import attr, base, deformer, unit
+from . import attr, baseclasses, unit
 
 # some helper routines
 
@@ -47,17 +46,10 @@ def _mmatrix(angle):
              (-2*math.sin(phi)*math.cos(phi),
               math.sin(phi)*math.sin(phi)-math.cos(phi)*math.cos(phi) ) )
 
-def _det(matrix):
-    return matrix[0][0]*matrix[1][1] - matrix[0][1]*matrix[1][0]
-
-# Exception
-
-class UndefinedResultError(ArithmeticError):
-    pass
 
 # trafo: affine transformations
 
-class trafo_pt(base.canvasitem, deformer.deformer):
+class trafo_pt(baseclasses.deformer):
 
     """affine transformation (coordinates in constructor in pts)
 
@@ -67,11 +59,10 @@ class trafo_pt(base.canvasitem, deformer.deformer):
 
     """
 
-    def __init__(self, matrix=((1,0),(0,1)), vector=(0,0)):
-        if _det(matrix)==0:
-            raise UndefinedResultError, "transformation matrix must not be singular" 
-        else:
-            self.matrix=matrix
+    def __init__(self, matrix=((1, 0), (0, 1)), vector=(0, 0)):
+        """Return trafo with given transformation matrix and vector.
+        """
+        self.matrix = matrix
         self.vector = vector
 
     def __mul__(self, other):
@@ -95,7 +86,7 @@ class trafo_pt(base.canvasitem, deformer.deformer):
 
             return trafo_pt(matrix=matrix, vector=vector)
         else:
-            raise NotImplementedError, "can only multiply two transformations"
+            raise NotImplementedError("can only multiply two transformations")
 
     def __str__(self):
         return "[%f %f %f %f %f %f]" % \
@@ -103,93 +94,82 @@ class trafo_pt(base.canvasitem, deformer.deformer):
                  self.matrix[0][1], self.matrix[1][1],
                  self.vector[0], self.vector[1] )
 
-    def outputPS(self, file):
+    def processPS(self, file, writer, context, registry):
         file.write("[%f %f %f %f %f %f] concat\n" % \
                     ( self.matrix[0][0], self.matrix[1][0],
                       self.matrix[0][1], self.matrix[1][1],
                       self.vector[0], self.vector[1] ) )
 
-    def outputPDF(self, file):
+    def processPDF(self, file, writer, context, registry):
         file.write("%f %f %f %f %f %f cm\n" % \
                     ( self.matrix[0][0], self.matrix[1][0],
                       self.matrix[0][1], self.matrix[1][1],
                       self.vector[0], self.vector[1] ) )
 
-    def bbox(self):
-        return None
-
-    def _apply(self, x, y):
-        """apply transformation to point (x,y) (coordinates in pts)"""
-        return (self.matrix[0][0]*x +
-                self.matrix[0][1]*y +
-                self.vector[0],
-                self.matrix[1][0]*x +
-                self.matrix[1][1]*y +
-                self.vector[1])
+    def apply_pt(self, x_pt, y_pt):
+        """apply transformation to point (x_pt, y_pt) in pts"""
+        return ( self.matrix[0][0]*x_pt + self.matrix[0][1]*y_pt + self.vector[0],
+                 self.matrix[1][0]*x_pt + self.matrix[1][1]*y_pt + self.vector[1] )
 
     def apply(self, x, y):
         # for the transformation we have to convert to points
-        tx, ty = self._apply(unit.topt(x), unit.topt(y))
+        tx, ty = self.apply_pt(unit.topt(x), unit.topt(y))
         return tx * unit.t_pt, ty * unit.t_pt
 
     def deform(self, path):
         return path.transformed(self)
 
     def inverse(self):
-        det = _det(self.matrix)                       # shouldn't be zero, but
-        try:
-          matrix = ( ( self.matrix[1][1]/det, -self.matrix[0][1]/det),
-                     (-self.matrix[1][0]/det,  self.matrix[0][0]/det)
-                   )
-        except ZeroDivisionError:
-           raise UndefinedResultError, "transformation matrix must not be singular" 
-        return trafo_pt(matrix=matrix) * \
-               trafo_pt(vector=(-self.vector[0], -self.vector[1]))
+        det = 1.0*(self.matrix[0][0]*self.matrix[1][1] - self.matrix[0][1]*self.matrix[1][0])
+        matrix = ( ( self.matrix[1][1]/det, -self.matrix[0][1]/det),
+                   (-self.matrix[1][0]/det,  self.matrix[0][0]/det) )
+        return ( trafo_pt(matrix=matrix) *
+                 trafo_pt(vector=(-self.vector[0], -self.vector[1])) )
 
     def mirrored(self, angle):
-        return mirror(angle)*self
+        return mirror(angle) * self
 
     def rotated_pt(self, angle, x=None, y=None):
-        return rotate_pt(angle, x, y)*self
+        return rotate_pt(angle, x, y) * self
 
     def rotated(self, angle, x=None, y=None):
-        return rotate(angle, x, y)*self
+        return rotate(angle, x, y) * self
 
     def scaled_pt(self, sx, sy=None, x=None, y=None):
-        return scale_pt(sx, sy, x, y)*self
+        return scale_pt(sx, sy, x, y) * self
 
     def scaled(self, sx, sy=None, x=None, y=None):
-        return scale(sx, sy, x, y)*self
+        return scale(sx, sy, x, y) * self
 
     def slanted_pt(self, a, angle=0, x=None, y=None):
-        return slant_pt(a, angle, x, y)*self
+        return slant_pt(a, angle, x, y) * self
 
     def slanted(self, a, angle=0, x=None, y=None):
-        return slant(a, angle, x, y)*self
+        return slant(a, angle, x, y) * self
 
     def translated_pt(self, x, y):
-        return translate_pt(x,y)*self
+        return translate_pt(x, y) * self
 
     def translated(self, x, y):
-        return translate(x, y)*self
+        return translate(x, y) * self
 
 
 class trafo(trafo_pt):
 
     """affine transformation"""
 
-    def __init__(self, matrix=((1,0),(0,1)), vector=(0,0)):
+    def __init__(self, matrix=((1,0), (0,1)), vector=(0, 0)):
         trafo_pt.__init__(self,
-                        matrix,
-                        (unit.topt(vector[0]), unit.topt(vector[1])))
-
+                          matrix, (unit.topt(vector[0]), unit.topt(vector[1])))
 
 #
 # some standard transformations 
 #
 
+identity = trafo()
+
 class mirror(trafo):
-    def __init__(self,angle=0):
+    def __init__(self, angle=0):
         trafo.__init__(self, matrix=_mmatrix(angle))
 
 
@@ -198,13 +178,10 @@ class rotate_pt(trafo_pt):
         vector = 0, 0
         if x is not None or y is not None:
             if x is None or y is None:
-                raise (UndefinedResultError, 
-                       "either specify both x and y or none of them")
+                raise TrafoException("either specify both x and y or none of them")
             vector=_rvector(angle, x, y)
 
-        trafo_pt.__init__(self,
-                       matrix=_rmatrix(angle),
-                       vector=vector)
+        trafo_pt.__init__(self, matrix=_rmatrix(angle), vector=vector)
 
 
 class rotate(trafo_pt):
@@ -212,59 +189,48 @@ class rotate(trafo_pt):
         vector = 0, 0 
         if x is not None or y is not None:
             if x is None or y is None:
-                raise (UndefinedResultError, 
-                       "either specify both x and y or none of them")
+                raise TrafoException("either specify both x and y or none of them")
             vector=_rvector(angle, unit.topt(x), unit.topt(y))
 
-        trafo_pt.__init__(self,
-                       matrix=_rmatrix(angle),
-                       vector=vector)
+        trafo_pt.__init__(self, matrix=_rmatrix(angle), vector=vector)
 
 
 class scale_pt(trafo_pt):
     def __init__(self, sx, sy=None, x=None, y=None):
-        sy = sy or sx
-        if not sx or not sy:
-            raise (UndefinedResultError, 
-                   "one scaling factor is 0")
-        vector = 0, 0 
+        if sy is None:
+            sy = sx
+        vector = 0, 0
         if x is not None or y is not None:
             if x is None or y is None:
-                raise (UndefinedResultError, 
-                       "either specify both x and y or none of them")
+                raise TrafoException("either specify both x and y or none of them")
             vector = (1-sx)*x, (1-sy)*y
-
-        trafo_pt.__init__(self, matrix=((sx,0),(0,sy)), vector=vector)
+        trafo_pt.__init__(self, matrix=((sx, 0), (0, sy)), vector=vector)
 
 
 class scale(trafo):
     def __init__(self, sx, sy=None, x=None, y=None):
-        sy = sy or sx
-        if not sx or not sy:
-            raise (UndefinedResultError, 
-                   "one scaling factor is 0")
-        vector = 0, 0 
+        if sy is None:
+            sy = sx
+        vector = 0, 0
         if x is not None or y is not None:
             if x is None or y is None:
-                raise (UndefinedResultError, 
-                       "either specify both x and y or none of them")
+                raise TrafoException("either specify both x and y or none of them")
             vector = (1-sx)*x, (1-sy)*y
-
-        trafo.__init__(self, matrix=((sx,0),(0,sy)), vector=vector)
+        trafo.__init__(self, matrix=((sx, 0), (0, sy)), vector=vector)
 
 
 class slant_pt(trafo_pt):
     def __init__(self, a, angle=0, x=None, y=None):
-        t = ( rotate_pt(-angle, x, y)*
-              trafo(matrix=((1, a), (0, 1)))*
+        t = ( rotate_pt(-angle, x, y) *
+              trafo(matrix=((1, a), (0, 1))) *
               rotate_pt(angle, x, y) )
         trafo_pt.__init__(self, t.matrix, t.vector)
 
 
 class slant(trafo):
     def __init__(self, a, angle=0, x=None, y=None):
-        t = ( rotate(-angle, x, y)*
-              trafo(matrix=((1, a), (0, 1)))*
+        t = ( rotate(-angle, x, y) *
+              trafo(matrix=((1, a), (0, 1))) *
               rotate(angle, x, y) )
         trafo.__init__(self, t.matrix, t.vector)
 
